
-- All Haskell function are "curried", which means that
-- ALL Haskell functions only take one parameter (even
-- when it looks like they take more than one).

-- Here is a function that seems to take two parameters.

f1 x y = x + y

-- Here is its type.

f1 :: Num a => a -> a -> a

-- This should be read the following way

f1 :: Num a => a -> (a -> a)

-- The function takes one input and returns another function 
-- (that also takes one input).

-- Here is a way to rewite f1 so that it explicitly takes
-- one parameter and returns a function that takes one parameter.
-- From Haskell's point of view f1 and f2 are the same. In fact
-- f1 is just "syntactic sugar" for f2.

f2 x = (\y -> x + y)   -- lambda expression


-- Another example. g1 is syntactic sugar for g2.
g1 x xs = x : xs
g2 x = (\xs -> x : xs)



-- Here is a function that seems to take three parameters.
h1 x y z = x + y + z
-- Next is an equivalent function that takes one parameter and
-- returns a curried function that seems to take two parameters.
h2 x = (\y z -> x + y + z)
-- Third is an equivalent function that takes one parameter and 
-- returns a function that takes one parameter that returns a 
-- function that takes one parameter. h1 is syntactic sugar for h3.
h3 x = (\y -> (\z -> x + y + z))
-- Here is an anonymous version of h3 (which we name h4).
h4 = (\x -> (\y -> (\z -> x + y + z)))
-- Notice the subtle difference between h3 and h4. h3 cannot
-- exist without its name, but the function in h4 can.
{- Try this in Hugs.
(\x -> (\y -> (\z -> x + y + z))) 2 3 4
-}
-- The anonymous function on the right hand side of h4 is a
-- valid function by itself. But the expression on the right
-- hand side of h3 is not a valid function (it has an unbound
-- variable x).


-- I just noticed that Hugs types h3 and h4 differently!
-- Load this file into Hugs and then ask for the type of
-- h3 and h4. They are not the same. I have no idea why.

{- Try in Hugs.
:t h3
:t h4
-}

-- Hugs does the same thing for the following two functions.
h5 x = x+1
h6 = (\x -> x+1)
{- Try in Hugs.
:t h5
:t h6
-}
