
-- Some higher order functions on lists.


-- apply a function to each element of a list
map' :: (a -> b) -> [a] -> [b]
map' f [] = []
map' f (x:xs) = (f x) : map' f xs

-- map sqrt [1..5]


-- select elements of a list that satisfy a boolean function
filter' :: (a -> Bool) -> [a] -> [a]
filter' p [] = []
filter' p (x:xs) | p x = x : (filter' p xs)
                 | otherwise = filter' p xs

-- filter (\x -> length x > 2) [[5,2,7], [ ], [0,1], [1..3]]


-- zip, then apply a function to each pair
zipWith' :: (a -> b -> c) -> [a] -> [b] -> [c]
zipWith' f [] _ = []
zipWith' f _ [] = []
zipWith' f (x:xs) (y:ys) = (f x y) : zipWith' f xs ys
-- zipWith' f (x:xs) (y:ys) = (f x y) : zipWith' f ys xs


-- zipWith (*) [2,3,4] [5,5,0]


-- returns a list containing elements from the front of the list
-- while the condition is satisfied.
takeWhile' :: (a -> Bool) -> [a] -> [a]
takeWhile' p [] = []
takeWhile' p (x:xs) | p x = x : (takeWhile' p xs)
                    | otherwise = takeWhile' p xs

-- takeWhile (<1000) ( iterate (\x -> 2*x) 2 )



-- "fold" a non-empty list starting at the right
foldr1' :: (a -> a -> a) -> [a] -> a
foldr1' f [x] = x
foldr1' f (x:xs) = f x (foldr1' f xs)


-- foldr1 (-) [1,2,3,4]  ==>  1-(2-(3-4))


-- "fold" a list starting at the right, using an initial value
foldr' :: (a -> b -> b) -> b -> [a] -> b
foldr' f z [] = z
foldr' f z (x:xs) = f x (foldr' f z xs)


-- foldr (-) 5 [1,2,3,4]  ==>   1-(2-(3-(4-5)))


-- Notice that
--     sum  = foldr (+)     0
--     and  = foldr (&&)    True
--   concat = foldr (++)    []
--    map f = foldr ((:).f) []
-- filter p = foldr (\x xs -> if p x then x:xs else xs) []


-- "fold" a non-empty list starting at the left
foldl1' :: (a -> a -> a) -> [a] -> a
foldl1' f [x] = x
foldl1' f (x:y:ys) = foldl1' f ((f x y):ys)


-- foldl1 (-) [1,2,3,4]  ==>  ((1-2)-3)-4


-- "fold" a list starting at the left, using an initial value
foldl' :: (b -> a -> b) -> b -> [a] -> b
foldl' f z [] = z
foldl' f z (x:xs) = foldl' f (f z x) xs


-- foldl (-) 1 [2,3,4,5]  ==>  (((1-2)-3)-4)-5
