
-- Consider this curried function.

f :: Int -> Bool -> Int -> Bool
f n b m = even (n + if b then m else 0)

-- This function seems to consume three parameters but 
-- in fact it consumes only one parameter and returns a 
-- function (that seems to consume two parameters).

-- Since f really consumes just one parameter, what do 
-- we get if we call it with only one parameter? We get 
-- back a (curried) function that seems to consume two 
-- parameters but really consumes just one parameter 
-- (and returns a function that consumes one parameter).

-- What if we call f with two parameters? We get back a 
-- function that consumes one parameter.

-- When we call a curried function with fewer parameters than 
-- what it seems to consume, we call that "partial application".
-- Partial application of a curried function always returns a 
-- new function. In the case of f, here are the types of the
-- results from calling f with diiferent numbers of parameters.

-- f :: Int -> Bool -> Int -> Bool
-- f 2      :: Bool -> Int -> Bool
-- f 2 True         :: Int -> Bool
-- f 2 True 3              :: Bool

{- Try in Hugs
:t f
:t f 2
:t f 2 True
:t f 2 True 3
-}

-- Try it in Hugs.
f1 = f 2
f2 = f 2 True
{-
f1 True 3
f2 3
f2 4
f1 False 4
f1 True  4
-}
